#!/usr/bin/python3
import sys
import smbus

#defines
LW09_REG_STATUS  = 0x00
LW09_REG_COMMAND = 0x01
LW09_REG_CONFIG  = 0x08
LW09_REG_ADDRESS = 0xFE

LW09_DALI_BROADCAST = 0xFE
LW09_DALI_GROUP     = 0x80
LW09_DALI_NOGROUP   = 0x00
LW09_DALI_MODE_CMD  = 0x01
LW09_DALI_MODE_DCAP = 0x00

LW09_DALI_DAPC_MIN = 0x00 	#0
LW09_DALI_DAPC_MAX = 0xFE	#254

address = 0x20	#7bit I2C-Address

i2c = smbus.SMBus(1)

def SendCommand(dali_device, cmd):
	values = [dali_device, cmd]
	ret = i2c.write_i2c_block_data(address, LW09_REG_COMMAND, values)
	return ret

#Get Address for LW09 (Datasheet Page 3, 4.3.2)
#group = 0 : single device
#group = 1 : group address
#num: device(0...63) / group(0...15) address 
#mode = 0: DACP mode (direct output)
#mode = 1: Command mode
def GetAddress(group, num, mode):
	temp = int(num) << 1	#Shift left
	adr = group | temp | mode
	return adr

#Device num:
#For single device: 0...63
#For group        : 0...15
#For broadcast    : LW09_DALI_GROUP and device 63
dacp = 200	#DACP value
device = 1	#DALI device address
adr = GetAddress(LW09_DALI_NOGROUP, device, LW09_DALI_MODE_DCAP)
ret = SendCommand(adr, dacp) #values from 0...254 (0x00...0xFE) for DCAP mode (dimming)

print (ret)